import {
    CartChangedError,
    CheckoutSelectors,
    CheckoutSettings,
    OrderRequestBody,
    PaymentMethod,
    Checkout,
} from '@bigcommerce/checkout-sdk';
import { memoizeOne } from '@bigcommerce/memoize';
import { compact, find, isEmpty, noop } from 'lodash';
import React, { Component, ReactNode } from 'react';
import { ObjectSchema } from 'yup';
import { connect } from 'react-redux'

import B3Checkout from '../b3checkout'
import MiniBC from '../minibc'
import { isMiniBCPayment }  from '../minibc/utils'
import { AnalyticsContextProps } from '@bigcommerce/checkout/analytics';
import { PaymentFormValues } from '@bigcommerce/checkout/payment-integration-api';
import { ChecklistSkeleton } from '@bigcommerce/checkout/ui';

import { withAnalytics } from '../analytics';
import { CheckoutContextProps, withCheckout } from '../checkout';
import {
    ErrorLogger,
    ErrorModal,
    ErrorModalOnCloseProps,
    isCartChangedError,
    isErrorWithType,
} from '../common/error';
import { EMPTY_ARRAY } from '../common/utility';
import { withLanguage, WithLanguageProps } from '../locale';
import { TermsConditionsType } from '../termsConditions';
import { storage } from '../b3checkout/utils'

import mapSubmitOrderErrorMessage, { mapSubmitOrderErrorTitle } from './mapSubmitOrderErrorMessage';
import mapToOrderRequestBody from './mapToOrderRequestBody';
import PaymentContext from './PaymentContext';
import PaymentForm from './PaymentForm';
import {
    getUniquePaymentMethodId,
    PaymentMethodId,
    PaymentMethodProviderType,
} from './paymentMethod';

export interface PaymentProps {
    errorLogger: ErrorLogger;
    isEmbedded?: boolean;
    isUsingMultiShipping?: boolean;
    checkEmbeddedSupport?(methodIds: string[]): void; // TODO: We're currently doing this check in multiple places, perhaps we should move it up so this check get be done in a single place instead.
    onCartChangedError?(error: CartChangedError): void;
    onFinalize?(): void;
    onFinalizeError?(error: Error): void;
    onReady?(): void;
    onSubmit?(): void;
    onSubmitError?(error: Error): void;
    onUnhandledError?(error: Error): void;
}

interface WithCheckoutPaymentProps {
    availableStoreCredit: number;
    cartUrl: string;
    defaultMethod?: PaymentMethod;
    finalizeOrderError?: Error;
    isInitializingPayment: boolean;
    isSubmittingOrder: boolean;
    isStoreCreditApplied: boolean;
    isTermsConditionsRequired: boolean;
    methods: PaymentMethod[];
    shouldExecuteSpamCheck: boolean;
    shouldLocaliseErrorMessages: boolean;
    submitOrderError?: Error;
    termsConditionsText?: string;
    termsConditionsUrl?: string;
    usableStoreCredit: number;
    applyStoreCredit(useStoreCredit: boolean): Promise<CheckoutSelectors>;
    clearError(error: Error): void;
    finalizeOrderIfNeeded(): Promise<CheckoutSelectors>;
    isPaymentDataRequired(): boolean;
    loadCheckout(): Promise<CheckoutSelectors>;
    loadPaymentMethods(): Promise<CheckoutSelectors>;
    submitOrder(values: OrderRequestBody): Promise<CheckoutSelectors>;
    storeHash: string;
    checkout: Checkout;
}

interface PaymentState {
    didExceedSpamLimit: boolean;
    isReady: boolean;
    selectedMethod?: PaymentMethod;
    shouldDisableSubmit: { [key: string]: boolean };
    shouldHidePaymentSubmitButton: { [key: string]: boolean };
    submitFunctions: { [key: string]: ((values: PaymentFormValues) => void) | null };
    validationSchemas: { [key: string]: ObjectSchema<Partial<PaymentFormValues>> | null };
}

interface B3Props {
    isB3User: boolean
    b3Methods: any
    checkoutSettings: any
}

class Payment extends Component<
    B3Props & PaymentProps & WithCheckoutPaymentProps & WithLanguageProps & AnalyticsContextProps,
    PaymentState
> {
    state: PaymentState = {
        didExceedSpamLimit: false,
        isReady: false,
        shouldDisableSubmit: {},
        shouldHidePaymentSubmitButton: {},
        validationSchemas: {},
        submitFunctions: {},
    };

    private getContextValue = memoizeOne(() => {
        return {
            disableSubmit: this.disableSubmit,
            setSubmit: this.setSubmit,
            setValidationSchema: this.setValidationSchema,
            hidePaymentSubmitButton: this.hidePaymentSubmitButton,
        };
    });

    get methods(): PaymentMethod[] {
        const {
            methods,
            isB3User,
            b3Methods,
        } = this.props

        const {
            paymentFilter
        } = B3Checkout.utils.util

        const getCurrency = (window as any).sessionStorage.getItem('currencySelector')
        const currencyMethod = (getCurrency !== 'undefined' && getCurrency !== '') ? JSON.parse(getCurrency) : ''
        if (currencyMethod && currencyMethod?.active_currency_code !== currencyMethod?.default_currency_code) {
            return methods
        }

        if (!isB3User) return methods.filter(method => method.id !== 'cheque')
        return paymentFilter(methods, b3Methods)
    }

    get isB2BExistingMethod() {
        const { defaultMethod } = this.props
        const isExistMethod = this.methods.some(({id}) => {(defaultMethod as any).id === id})
        return isExistMethod
    }

    async componentDidMount(): Promise<void> {
        const {
            finalizeOrderIfNeeded,
            loadPaymentMethods,
            onFinalize = noop,
            onFinalizeError = noop,
            onReady = noop,
            onUnhandledError = noop,
            checkout,
            isB3User,
            storeHash,
            usableStoreCredit,
            methods,
            defaultMethod,
            analyticsTracker
        } = this.props;

        const { selectedMethod } = this.state;

        analyticsTracker.selectedPaymentMethod((selectedMethod || defaultMethod)?.config.displayName);

        if (usableStoreCredit) {
            this.handleStoreCreditChange(true);
        }

        try {
            await loadPaymentMethods();
            await B3Checkout.payment.init()
        } catch (error) {
            onUnhandledError(error);
        }

        try {
            const state = await finalizeOrderIfNeeded();
            const order = state.data.getOrder();

            const quoteId = storage.quoteCheckoutId.getValue() || ''
            if ((isB3User || quoteId) && checkout?.orderId)  {
                await B3Checkout.utils.payment.paymentsRefresh(methods)
                const isInvoice = await B3Checkout.payment.paySubmit(checkout, storeHash, checkout)
                if (isInvoice) return
            }
            onFinalize(order?.orderId);
        } catch (error) {
            if (isErrorWithType(error) && error.type !== 'order_finalization_not_required') {
                onFinalizeError(error);
            }
        }

        window.addEventListener('beforeunload', this.handleBeforeUnload);
        
        if (isB3User && !isEmpty(this.methods) && !this.isB2BExistingMethod) { 
            this.setSelectedMethod(this.methods[0])
        }

        this.setState({ isReady: true });
        onReady();
    }

    componentDidUpdate(): void {
        const {
            checkEmbeddedSupport = noop,
        } = this.props;

        checkEmbeddedSupport(this.methods.map(({ id }) => id));
    }

    componentWillUnmount(): void {
        window.removeEventListener('beforeunload', this.handleBeforeUnload);
    }

    render(): ReactNode {
        const {
            defaultMethod,
            isInitializingPayment,
            isUsingMultiShipping,
            ...rest
        } = this.props;

        const {
            didExceedSpamLimit,
            isReady,
            selectedMethod = defaultMethod,
            shouldDisableSubmit,
            validationSchemas,
            shouldHidePaymentSubmitButton,
        } = this.state;

        const uniqueSelectedMethodId =
            selectedMethod && getUniquePaymentMethodId(selectedMethod.id, selectedMethod.gateway);

        return (
            <PaymentContext.Provider value={this.getContextValue()}>
                <ChecklistSkeleton isLoading={!isReady}>
                    {!isEmpty(this.methods) && defaultMethod && (
                        <PaymentForm
                            {...rest}
                            defaultGatewayId={defaultMethod.gateway}
                            defaultMethodId={defaultMethod.id}
                            didExceedSpamLimit={didExceedSpamLimit}
                            isInitializingPayment={isInitializingPayment}
                            isUsingMultiShipping={isUsingMultiShipping}
                            methods={this.methods}
                            onMethodSelect={this.setSelectedMethod}
                            onStoreCreditChange={this.handleStoreCreditChange}
                            onSubmit={this.handleSubmit}
                            onUnhandledError={this.handleError}
                            selectedMethod={selectedMethod}
                            shouldDisableSubmit={
                                (uniqueSelectedMethodId &&
                                    shouldDisableSubmit[uniqueSelectedMethodId]) ||
                                undefined
                            }
                            shouldHidePaymentSubmitButton={
                                (uniqueSelectedMethodId &&
                                    shouldHidePaymentSubmitButton[uniqueSelectedMethodId]) ||
                                undefined
                            }
                            validationSchema={
                                (uniqueSelectedMethodId &&
                                    validationSchemas[uniqueSelectedMethodId]) ||
                                undefined
                            }
                        />
                    )}
                </ChecklistSkeleton>

                {this.renderOrderErrorModal()}
                {this.renderEmbeddedSupportErrorModal()}
            </PaymentContext.Provider>
        );
    }

    private renderOrderErrorModal(): ReactNode {
        const { finalizeOrderError, language, shouldLocaliseErrorMessages, submitOrderError, methods } =
            this.props;

        // FIXME: Export correct TS interface
        const error: any = submitOrderError || finalizeOrderError;

        if (
            !error ||
            error.type === 'order_finalization_not_required' ||
            error.type === 'payment_cancelled' ||
            error.type === 'payment_invalid_form' ||
            error.type === 'spam_protection_not_completed' ||
            error.type === 'invalid_hosted_form_value'
        ) {
            return null;
        }

        if ((error.message.indexOf("We're experiencing difficulty processing your transaction.") >= 0 ||
            error.message.indexOf("Your session has expired. Please log in again.") >= 0  || 
            error.message.indexOf("Vaulting service is currently not available.") >= 0) &&
            error.name === 'RequestError' &&
            isMiniBCPayment(methods)) return null;

        return (
            <ErrorModal
                error={error}
                message={mapSubmitOrderErrorMessage(
                    error,
                    language.translate.bind(language),
                    shouldLocaliseErrorMessages,
                )}
                onClose={this.handleCloseModal}
                title={mapSubmitOrderErrorTitle(error, language.translate.bind(language))}
            />
        );
    }

    private renderEmbeddedSupportErrorModal(): ReactNode {
        const {
            checkEmbeddedSupport = noop,
        } = this.props;

        try {
            checkEmbeddedSupport(this.methods.map(({ id }) => id));
        } catch (error) {
            if (error instanceof Error) {
                return <ErrorModal error={error} onClose={this.handleCloseModal} />;
            }
        }

        return null;
    }

    private disableSubmit: (method: PaymentMethod, disabled?: boolean) => void = (
        method,
        disabled = true,
    ) => {
        const uniqueId = getUniquePaymentMethodId(method.id, method.gateway);
        const { shouldDisableSubmit } = this.state;

        if (shouldDisableSubmit[uniqueId] === disabled) {
            return;
        }

        this.setState({
            shouldDisableSubmit: {
                ...shouldDisableSubmit,
                [uniqueId]: disabled,
            },
        });
    };

    private hidePaymentSubmitButton: (method: PaymentMethod, disabled?: boolean) => void = (
        method,
        disabled = true,
    ) => {
        const uniqueId = getUniquePaymentMethodId(method.id, method.gateway);
        const { shouldHidePaymentSubmitButton } = this.state;

        if (shouldHidePaymentSubmitButton[uniqueId] === disabled) {
            return;
        }

        this.setState({
            shouldHidePaymentSubmitButton: {
                ...shouldHidePaymentSubmitButton,
                [uniqueId]: disabled,
            },
        });
    };

    // tslint:disable:cyclomatic-complexity
    private handleBeforeUnload: (event: BeforeUnloadEvent) => string | undefined = (event) => {
        const { defaultMethod, isSubmittingOrder, language } = this.props;
        const { selectedMethod = defaultMethod } = this.state;

        // TODO: Perhaps there is a better way to handle `adyen`, `afterpay`, `amazonpay`,
        // `checkout.com`, `converge`, `sagepay`, `stripev3` and `sezzle`. They require
        //  a redirection to another website during the payment flow but are not
        //  categorised as hosted payment methods.
        if (
            !isSubmittingOrder ||
            !selectedMethod ||
            selectedMethod.type === PaymentMethodProviderType.Hosted ||
            selectedMethod.type === PaymentMethodProviderType.PPSDK ||
            selectedMethod.id === PaymentMethodId.AmazonPay ||
            selectedMethod.id === PaymentMethodId.CBAMPGS ||
            selectedMethod.id === PaymentMethodId.Checkoutcom ||
            selectedMethod.id === PaymentMethodId.CheckoutcomGooglePay ||
            selectedMethod.id === PaymentMethodId.Converge ||
            selectedMethod.id === PaymentMethodId.Humm ||
            selectedMethod.id === PaymentMethodId.Laybuy ||
            selectedMethod.id === PaymentMethodId.Opy ||
            selectedMethod.id === PaymentMethodId.Quadpay ||
            selectedMethod.id === PaymentMethodId.SagePay ||
            selectedMethod.id === PaymentMethodId.Sezzle ||
            selectedMethod.id === PaymentMethodId.WorldpayAccess ||
            selectedMethod.id === PaymentMethodId.Zip ||
            selectedMethod.gateway === PaymentMethodId.AdyenV2 ||
            selectedMethod.gateway === PaymentMethodId.AdyenV2GooglePay ||
            selectedMethod.gateway === PaymentMethodId.AdyenV3 ||
            selectedMethod.gateway === PaymentMethodId.AdyenV3GooglePay ||
            selectedMethod.gateway === PaymentMethodId.Afterpay ||
            selectedMethod.gateway === PaymentMethodId.Clearpay ||
            selectedMethod.gateway === PaymentMethodId.Checkoutcom ||
            selectedMethod.gateway === PaymentMethodId.Mollie ||
            selectedMethod.gateway === PaymentMethodId.StripeV3
        ) {
            return;
        }

        const message = language.translate('common.leave_warning');

        event.returnValue = message;

        return message;
    };

    private handleCloseModal: (event: Event, props: ErrorModalOnCloseProps) => Promise<void> =
        async (_, { error }) => {
            if (!error) {
                return;
            }

            const { cartUrl, clearError, loadCheckout } = this.props;
            const { type: errorType } = error as any; // FIXME: Export correct TS interface

            if (
                errorType === 'provider_fatal_error' ||
                errorType === 'order_could_not_be_finalized_error'
            ) {
                window.location.replace(cartUrl || '/');
            }

            if (errorType === 'tax_provider_unavailable') {
                window.location.reload();
            }

            if (errorType === 'cart_consistency') {
                await loadCheckout();
            }

            if (isErrorWithType(error) && error.body) {
                const { body, headers, status } = error;

                if (body.type === 'provider_error' && headers.location) {
                    window.top?.location.assign(headers.location);
                }

                // Reload the checkout object to get the latest `shouldExecuteSpamCheck` value,
                // which will in turn make `SpamProtectionField` visible again.
                // NOTE: As a temporary fix, we're checking the status code instead of the error
                // type because of an issue with Nginx config, which causes the server to return
                // HTML page instead of JSON response when there is a 429 error.
                if (
                    status === 429 ||
                    body.type === 'spam_protection_expired' ||
                    body.type === 'spam_protection_failed'
                ) {
                    this.setState({ didExceedSpamLimit: true });

                    await loadCheckout();
                }
            }

            clearError(error);
        };

    private handleStoreCreditChange: (useStoreCredit: boolean) => void = async (useStoreCredit) => {
        const { applyStoreCredit, onUnhandledError = noop } = this.props;

        try {
            await applyStoreCredit(useStoreCredit);
        } catch (e) {
            onUnhandledError(e);
        }
    };

    private handleError: (error: Error) => void = (error: Error) => {
        const { onUnhandledError = noop, errorLogger } = this.props;

        const { type } = error as any;

        if (type === 'unexpected_detachment') {
            errorLogger.log(error);

            return;
        }

        return onUnhandledError(error);
    };

    private handleSubmit: (values: PaymentFormValues) => void = async (values) => {
        //console.log("submit PROPSs - ", this.props)
        const {
            defaultMethod,
            loadPaymentMethods,
            loadCheckout,
            isPaymentDataRequired,
            onCartChangedError = noop,
            onSubmit = noop,
            onSubmitError = noop,
            submitOrder,
            analyticsTracker,
            isB3User,
            onUnhandledError = noop,
            storeHash,
            checkoutSettings,
            checkout,
            methods,
        } = this.props;

        const { selectedMethod = defaultMethod, submitFunctions } = this.state;

        analyticsTracker.clickPayButton({shouldCreateAccount: values.shouldCreateAccount});

        const customSubmit =
            selectedMethod &&
            submitFunctions[getUniquePaymentMethodId(selectedMethod.id, selectedMethod.gateway)];

        if (customSubmit) {
            return customSubmit(values);
        }

        const {
            checkoutPaymentAdditionalEnableExtra,
            checkoutPaymentAdditionalExtraFieldsRequired,
            checkoutPaymentAdditionalExtraFields,
            checkoutPaymentPurchaseEnableExtra,
            checkoutPaymentPurchaseExtraFields,
            checkoutPaymentPurchaseExtraFieldsRequired,
        } = checkoutSettings

        const b3PoNumberRequired = () => {
            return isB3User &&
            selectedMethod?.id === 'cheque' &&
            +checkoutPaymentPurchaseEnableExtra.value === 1 &&
            +checkoutPaymentPurchaseExtraFieldsRequired.value === 1
        }

        const {
            storage: {
                B3PoNumber,
                B3PaymentsAdditionalTextFieldValue,
            },
        } = B3Checkout.utils

        const requiredFieldTip = (fieldName: string) => {
            return new Error(`${B3Checkout.locales.tips['b3.field.required']} ${fieldName}`)
        }

        if (storage.IPCheckout.getValue() !== '1' && b3PoNumberRequired() && !B3PoNumber.getValue()) return onUnhandledError(requiredFieldTip(checkoutPaymentPurchaseExtraFields.value))

        const b3PaymentsAdditionalTextFieldRequired = () => {
            return isB3User &&
                +checkoutPaymentAdditionalEnableExtra.value === 1 &&
                +checkoutPaymentAdditionalExtraFieldsRequired.value === 1
        }

        if (storage.IPCheckout.getValue() !== '1' && b3PaymentsAdditionalTextFieldRequired() && !B3PaymentsAdditionalTextFieldValue.getValue()) return onUnhandledError(requiredFieldTip(checkoutPaymentAdditionalExtraFields.value))
        
        try {
            if (isB3User) await B3Checkout.extrasFiles.validateOrderExtraFields(values)
            if (isB3User) await B3Checkout.utils.payment.paymentsRefresh(methods)
        
            // Third-party payment save order val
            B3Checkout.extrasFiles.saveOrderExtraVlaues(values)

            B3Checkout.extrasFiles.saveAddressExtrasFiles()

            // const state = await submitOrder(mapToOrderRequestBody(values, isPaymentDataRequired()));
            // const order = state.data.getOrder();

            let order = null
            try {
                //console.log("submitOrder! ", values)

                const { data } = await submitOrder(mapToOrderRequestBody(values, isPaymentDataRequired()));
                order = data.getOrder()

            } catch (error) {
                //should receive a payment error from BC for cyberSource
                //console.log("BC SubmitOrder Error")
                //console.warn(error)
                if (!isMiniBCPayment(methods)) {
                    onSubmitError(error);
                    return
                }
            }

            //**** MINIBC CALLS ****//
            let minibcSuccess = true
            const savePaymentEle = document.getElementById('shouldSaveInstrument') as any;
            const savePayment = savePaymentEle ? savePaymentEle.value : false;

            if (!order && isMiniBCPayment(methods)) {
                this.setState({ isReady: false });
                const { data } = await loadCheckout()
                order = data.getOrder()

                //console.log(order)
                if (!order) throw Error("BC order not created")
                
                try {

                    let selecteMiniBCMethod : any = null
                    let authToken = ""
                    let custToken = "" 

                    //console.log("BC selectedMethod - ", selectedMethod)
                    const miniBCMethods : any = await MiniBC.request.validateCheckout(checkout)
                    //console.log("MINIBC methods - ", JSON.stringify(miniBCMethods))
                    
                    if (miniBCMethods?.data?.methods && selectedMethod) {
                        for (let miniBCMethod of miniBCMethods.data.methods) {
                            if (miniBCMethod.id === selectedMethod.id)  {
                                selecteMiniBCMethod = miniBCMethod
                                selecteMiniBCMethod["intent"] = miniBCMethods.data.intent
                                selecteMiniBCMethod["savePayment"] = savePayment
                            }
                        }
                    }
                    else throw Error("Could Not Initialize MiniBC Payment Method")
                    const miniBCPayment = await MiniBC.request.initPayment(checkout?.id, selecteMiniBCMethod)
                    //console.log("MINIBC payment - ", JSON.stringify(miniBCPayment))

                    if (miniBCPayment?.data?.success) {
                        authToken = miniBCPayment.data.auth_token
                        custToken = miniBCPayment.data.client_token
                    }
                    else throw Error("Could Not Initialize MiniBC Payment")

                    //console.log("custToken - ", custToken)

                    const miniBCOrder = await MiniBC.request.initOrder(checkout?.id, selecteMiniBCMethod, values)
                    //console.log("MINIBC order - ", JSON.stringify(miniBCOrder))

                    if (!miniBCOrder?.data?.success) throw Error("Could Not Initialize MiniBC Payment")
                        
                    const miniBCProcessPayment = await MiniBC.request.processPayment(order, authToken, selecteMiniBCMethod, values, custToken)
                    //console.log("MINIBC miniBCProcessPayment - ", JSON.stringify(miniBCProcessPayment))
                    if (!miniBCProcessPayment?.data?.success) throw new Error(`Could not make payment - ${miniBCProcessPayment?.errors && miniBCProcessPayment?.errors[0] ? miniBCProcessPayment.errors[0] : ''}`)

                    await MiniBC.request.confirmOrder(checkout?.id)
                    //const miniBCConfirmOrder = await MiniBC.request.confirmOrder(checkout?.id)
                    //console.log("MINIBC miniBCConfirmOrder - ", JSON.stringify(miniBCConfirmOrder))
                } catch (err) {

                    minibcSuccess = false
                    console.warn("MINIBC ERROR")
                    this.setState({ isReady: true });
                    if (err?.response?.data?.debug) onUnhandledError(new Error(err?.response?.data?.debug));
                    else onUnhandledError(err);

                }

                this.setState({ isReady: true });
                //**** END MINIBC CALLS ****//
            }

            const quoteId = storage.quoteCheckoutId.getValue() || ''
            if ((isB3User || quoteId) && order?.orderId)  {
                const isInvoice = await B3Checkout.payment.paySubmit(order, storeHash, checkout)
                if (isInvoice) return
            }
            analyticsTracker.paymentComplete();

            //console.log(minibcSuccess)
            minibcSuccess && onSubmit();
        } catch (error) {
            console.warn("Submit Error ", error)
            this.setState({ isReady: true });
            analyticsTracker.paymentRejected();

            if (isErrorWithType(error) && error.type === 'payment_method_invalid') {
                return loadPaymentMethods();
            }

            if (isCartChangedError(error)) {
                return onCartChangedError(error);
            }

            onSubmitError(error);
        }
    };

    private setSelectedMethod: (method?: PaymentMethod) => void = (method) => {
        const { analyticsTracker } = this.props;
        const { selectedMethod } = this.state;

        if (selectedMethod === method) {
            return;
        }

        analyticsTracker.selectedPaymentMethod(method?.config.displayName);

        this.setState({ selectedMethod: method });
    };

    private setSubmit: (
        method: PaymentMethod,
        fn: (values: PaymentFormValues) => void | null,
    ) => void = (method, fn) => {
        const uniqueId = getUniquePaymentMethodId(method.id, method.gateway);
        const { submitFunctions } = this.state;

        if (submitFunctions[uniqueId] === fn) {
            return;
        }

        this.setState({
            submitFunctions: {
                ...submitFunctions,
                [uniqueId]: fn,
            },
        });
    };

    private setValidationSchema: (
        method: PaymentMethod,
        schema: ObjectSchema<Partial<PaymentFormValues>> | null,
    ) => void = (method, schema) => {
        const uniqueId = getUniquePaymentMethodId(method.id, method.gateway);
        const { validationSchemas } = this.state;

        if (validationSchemas[uniqueId] === schema) {
            return;
        }

        this.setState({
            validationSchemas: {
                ...validationSchemas,
                [uniqueId]: schema,
            },
        });
    };
}

export function mapToPaymentProps({
        checkoutService,
        checkoutState,
}: CheckoutContextProps): WithCheckoutPaymentProps | null {
    const {
        data: {
            getCheckout,
            getConfig,
            getCustomer,
            getConsignments,
            getOrder,
            getPaymentMethod,
            getPaymentMethods,
            isPaymentDataRequired,
        },
        errors: { getFinalizeOrderError, getSubmitOrderError },
        statuses: { isInitializingPayment, isSubmittingOrder },
    } = checkoutState;

    const checkout = getCheckout();
    const config = getConfig();
    const customer = getCustomer();
    const consignments = getConsignments();
    const { isComplete = false } = getOrder() || {};
    let methods = getPaymentMethods() || EMPTY_ARRAY;

    // TODO: In accordance with the checkout team, this functionality is temporary and will be implemented in the backend instead.
    if (customer?.isStripeLinkAuthenticated) {
        const stripeUpePaymentMethod = methods.filter(method =>
            method.id === 'card' && method.gateway === PaymentMethodId.StripeUPE
        );

        methods = stripeUpePaymentMethod.length ? stripeUpePaymentMethod : methods;
    }


    if (!checkout || !config || !customer || isComplete) {
        return null;
    }

    const {
        enableTermsAndConditions: isTermsConditionsEnabled,
        features,
        orderTermsAndConditionsType: termsConditionsType,
        orderTermsAndConditions: termsCondtitionsText,
        orderTermsAndConditionsLink: termsCondtitionsUrl,
    } = config.checkoutSettings as CheckoutSettings & { orderTermsAndConditionsLocation: string };

    const {
        storeHash,
    } = config.storeProfile

    const isTermsConditionsRequired = isTermsConditionsEnabled;
    const selectedPayment = find(checkout.payments, {
        providerType: PaymentMethodProviderType.Hosted,
    });

    const { isStoreCreditApplied } = checkout;

    let selectedPaymentMethod;
    let filteredMethods;

    filteredMethods = methods.filter((method: PaymentMethod) => {
        if (method.id === PaymentMethodId.Bolt && method.initializationData) {
            return !!method.initializationData.showInCheckout;
        }

        return true;
    });

    if (consignments && consignments.length > 1) {
        const multiShippingIncompatibleMethodIds: string[] = [
            PaymentMethodId.AmazonPay,
        ];

        filteredMethods = methods.filter((method: PaymentMethod) => {
            return multiShippingIncompatibleMethodIds.indexOf(method.id) === -1;
        });
    }

    if (selectedPayment) {
        selectedPaymentMethod = getPaymentMethod(
            selectedPayment.providerId,
            selectedPayment.gatewayId,
        );
        filteredMethods = selectedPaymentMethod
            ? compact([selectedPaymentMethod])
            : filteredMethods;
    } else {
        selectedPaymentMethod = find(filteredMethods, {
            config: { hasDefaultStoredInstrument: true },
        });
        // eslint-disable-next-line no-self-assign
        filteredMethods = filteredMethods;
    }

    return {
        applyStoreCredit: checkoutService.applyStoreCredit,
        availableStoreCredit: customer.storeCredit,
        cartUrl: config.links.cartLink,
        clearError: checkoutService.clearError,
        defaultMethod: selectedPaymentMethod || filteredMethods[0],
        finalizeOrderError: getFinalizeOrderError(),
        finalizeOrderIfNeeded: checkoutService.finalizeOrderIfNeeded,
        loadCheckout: checkoutService.loadCheckout,
        isInitializingPayment: isInitializingPayment(),
        isPaymentDataRequired,
        isStoreCreditApplied,
        isSubmittingOrder: isSubmittingOrder(),
        isTermsConditionsRequired,
        loadPaymentMethods: checkoutService.loadPaymentMethods,
        methods: filteredMethods,
        shouldExecuteSpamCheck: checkout.shouldExecuteSpamCheck,
        shouldLocaliseErrorMessages:
            features['PAYMENTS-6799.localise_checkout_payment_error_messages'],
        submitOrder: checkoutService.submitOrder,
        submitOrderError: getSubmitOrderError(),
        termsConditionsText: isTermsConditionsRequired && termsConditionsType === TermsConditionsType.TextArea ?
            termsCondtitionsText :
            undefined,
        termsConditionsUrl: isTermsConditionsRequired && termsConditionsType === TermsConditionsType.Link ?
            termsCondtitionsUrl :
            undefined,
        usableStoreCredit: checkout.grandTotal > 0 ?
            Math.min(checkout.grandTotal, customer.storeCredit || 0) : 0,
        storeHash,
        checkout,
    };
}

const mapStateToProps = (state: {
    payment: { methods: any }
    global: { isB3User: boolean, checkoutSettings: any }
}) => {
    const {
        global: {
            isB3User,
            checkoutSettings,
        },
        payment: {
            methods: b3Methods
        }
    } = state

    return {
        isB3User,
        b3Methods,
        checkoutSettings,
    }
}

export default connect(mapStateToProps)(withAnalytics(withLanguage(withCheckout(mapToPaymentProps)(Payment))));
